#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "peri.h"
#include "usbdrv.h"

#define RQ_SET_GREEN       0
#define RQ_SET_YELLOW      1
#define RQ_SET_RED         2
#define RQ_SET_BLUE        3
#define RQ_GET_GREEN       4
#define RQ_GET_YELLOW      5
#define RQ_GET_RED         6
#define RQ_GET_BLUE        7

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint8_t switch_state;

    if (rq->bRequest == RQ_SET_GREEN)
    {
        uint8_t led_state = rq->wValue.bytes[0];
        set_green_value(led_state);
        return 0;
    }

    
    else if (rq->bRequest == RQ_SET_YELLOW)
    {
        uint8_t led_state = rq->wValue.bytes[0];
        set_yellow_value(led_state);
        return 0;
    }

    
    else if (rq->bRequest == RQ_SET_RED)
    {
        uint8_t led_state = rq->wValue.bytes[0];
        set_red_value(led_state);
        return 0;
    }

    
    else if (rq->bRequest == RQ_SET_BLUE)
    {
        uint8_t led_state = rq->wValue.bytes[0];
        set_blue_value(led_state);
        return 0;
    }

    else if (rq->bRequest == RQ_GET_GREEN)
    {
        switch_state = SWGREEN_PRESSED();

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }
    
    
    else if (rq->bRequest == RQ_GET_YELLOW)
    {
        switch_state = SWYELLOW_PRESSED();

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }

    
    else if (rq->bRequest == RQ_GET_RED)
    {
        switch_state = SWRED_PRESSED();

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }

    
    else if (rq->bRequest == RQ_GET_BLUE)
    {
        switch_state = SWBLUE_PRESSED();

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }

    /* default for not implemented requests: return no data back to host */
    return 0;
}

/* ------------------------------------------------------------------------- */
int main(void)
{
    init_peri();

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
